@@ -43,24 +43,20 @@ Known Issues:
     1) Multi-core compilation on Windows not supported
     2) Won't work on Windows 2000 or windows where home directory
        has spaces -- if we use ~ the build command will fail.
-
-Current Goals:
-    1) Get
 '''
 
 from optparse import OptionParser, OptionGroup #note: deprecated in Python27, use argparse
 from mozrunner import Runner, FirefoxRunner
-from mozrunner import FirefoxProfile
 import simplejson, urllib
 import ximport
-
 from types import *
-from utils import hgId, captureStdout
+from utils import hgId, captureStdout, increment_day
 import os, sys, subprocess, string, re, tempfile, shlex, glob, shutil, datetime, multiprocessing
+from time import gmtime, strftime
 
 #Global Variables
 showMakeData = 0
-progVersion="0.4.4"
+progVersion="0.4.5"
 
 class Builder():
     def __init__(self, makeCommand=["make","-f","client.mk","build"] , shellCacheDir=os.path.join(os.path.expanduser("~"), "moz-commitbuilder-cache"), cores=1, repoURL="http://hg.mozilla.org/mozilla-central",clean=False, mozconf=None):
@@ -75,6 +71,7 @@ class Builder():
         self.mozconf = mozconf
         self.objdir = os.path.join(self.repoPath,"obj-ff-dbg")
 
+        #Create directories we need
         if not os.path.exists(shellCacheDir):
             os.mkdir(shellCacheDir)
         if not os.path.exists(self.confDir):
@@ -103,22 +100,16 @@ class Builder():
         except:
             print "Woops, something went wrong!"
             quit()
+
+        #Ensure that changesetTip actually contains something
         if changesetTip:
             return changesetTip
         else:
             print "Couldn't get the tip changeset."
 
-    def increment_day(self, date):
-        #TODO: MOVE TO UTILS. Increments a date string.
-        s = date.split("-")
-        delta = datetime.timedelta(days=1)
-        nextDate = datetime.date(int(s[0]),int(s[1]),int(s[2])) + delta
-        return str(nextDate)
-
     def changesetFromDay(self, date, oldest=True):
-        # Gets first changeset from a given date
-        # NOTE: USES PUSHLOG
-        nextDate = self.increment_day(date)
+        # Gets first changeset from a given date via pushlog
+        nextDate = increment_day(date)
         pushlog_url = "http://hg.mozilla.org/mozilla-central/json-pushes?startdate="+date+"&enddate="+nextDate
         pushlog_json = simplejson.load(urllib.urlopen(pushlog_url))
         sorted_keys = sorted(map(int,pushlog_json.keys()))
@@ -143,8 +134,7 @@ class Builder():
         return False
 
     def getTrunk(self, makeClean=False):
-        #Get local trunk
-        #Delete old trunk if set to "clean" mode
+        #Get or update local trunk -- makeclean means delete old trunk and fetch fresh
         if makeClean:
             print "Making clean trunk environment..."
             try:
@@ -165,27 +155,36 @@ class Builder():
 
     def mozconfigure(self):
         #Set mozconfig settings
-        #See if we are told to use an externally customized one
         print "\nConfiguring mozconfig:"
-        if self.mozconf:
-            os.environ['MOZCONFIG']=self.mozconf
-            return
 
-        #Make mozconfig
+        #Make a place to put our mozconfig
         mozconfig_path = os.path.join(self.confDir, 'config-default')
         if os.path.exists(mozconfig_path):
             os.unlink(mozconfig_path)
 
+        #Using external mozconfig
+        if self.mozconf:
+            #Copy their mozconfig, but use our own objdir param
+            #Also cores. Don't break the flag's functionality!
+            shutil.copy(self.mozconf,mozconfig_path)
+            f=open(mozconfig_path, 'a')
+            f.write('mk_add_options MOZ_OBJDIR=@TOPSRCDIR@/obj-ff-dbg\n')
+            if sys.platform != "win32" or sys.platform != "cygwin":
+                f.write('mk_add_options MOZ_MAKE_FLAGS="-s -j '+str(self.cores)+'"')
+            os.environ['MOZCONFIG']=mozconfig_path
+            return
+
+        #Using our custom mozconfig
         f=open(mozconfig_path, 'w')
-        #Ensure we know where to find our built stuff by using a custom mozconfig
         f.write('mk_add_options MOZ_OBJDIR=@TOPSRCDIR@/obj-ff-dbg\n')
+
+        #According to Jesse, these are the fastest (build-wise) options
         f.write('ac_add_options --disable-optimize\n')
         f.write('ac_add_options --enable-debug\n')
         f.write('ac_add_options --enable-tests\n')
 
-        #HACK :/
         if sys.platform == "win32" or sys.platform == "cygwin":
-            print "Windows detected, compiling with 1 core."
+            print "Windows detected, multicore support disabled. Compiling with 1 core..."
             f.write("ac_add_options --with-windows-version=600\n")
             f.write("ac_add_options --enable-application=browser\n")
         else:
@@ -198,18 +197,14 @@ class Builder():
         os.environ['MOZCONFIG']=mozconfig_path
 
     def importAndBisect(self, good, bad, testcondition=None, args_for_condition=[]):
-        #convenience function for API use only
+        #Convenience function for API use only
         conditionscript = ximport.importRelativeOrAbsolute(testcondition)
         self.bisect(good,bad, testcondition=conditionscript, args_for_condition=args_for_condition)
 
-
-
     def bisect(self,good,bad, testcondition=None, args_for_condition=[]):
         #Call hg bisect with initial params, set up building environment (mozconfig)
-        #testfile is an external file, testpath is an actual mochitest
 
         #Support for using dates
-        #EXPERIMENTAL
         badDate = re.search(r'(\d\d\d\d-\d\d-\d\d)',good)
         goodDate = re.search(r'(\d\d\d\d-\d\d-\d\d)',bad)
         if badDate != None and goodDate != None:
@@ -218,9 +213,19 @@ class Builder():
             good = self.changesetFromDay(goodDate) #Get first changeset from this day
             bad = self.changesetFromDay(badDate, oldest=False) #Get last changeset from that day
 
+            if not (bad and good):
+                print "Invalid date range."
+                quit()
+
+            #Since they entered dates, lets give them info about the changeset range
+            print "Bisecting on changeset range " + str(good)[:12] + " to " + str(bad)[:12]
+
+        #Get the actual changesets that we will be using
         good = hgId(good, self.hgPrefix)
         bad = hgId(bad, self.hgPrefix)
-        if good and bad and self.validate(good, bad): #valid commit numbers, do the bisection!
+
+        if good and bad and self.validate(good, bad):
+            #Valid changesets, so do the bisection
             subprocess.call(self.hgPrefix+["bisect","--reset"])
 
             setUpdate = captureStdout(self.hgPrefix+["up",bad])
@@ -231,34 +236,53 @@ class Builder():
             print str(setBad)
             print str(setGood)
 
-            # Check if we should terminate early because the bisector exited?
-            string_to_parse = str(setGood)
-            traceback_flag = string_to_parse.find("Not all ancestors")
-            if traceback_flag > -1:
-                #TODO: Bisect from ancestor changeset rather than exiting early
-                quit()
-            traceback_flag = string_to_parse.find("The first bad revision is")
-            if traceback_flag > -1:
-                quit()
+            self.check_done(setGood)
 
             # Set mozconfig
-            # Call recursive bisection!
             self.mozconfigure()
+
+            # Call recursive bisection!
             self.bisectRecurse(testcondition=testcondition, args_for_condition=args_for_condition)
 
         else:
             print "Invalid values. Please check your changeset revision numbers."
 
+
+    def check_done(self, doneString):
+       # Check if we should terminate early because the bisector exited?
+        string_to_parse = str(doneString)
+
+        branch_unaware_flag = string_to_parse.find("Not all ancestors")
+        traceback_flag = string_to_parse.find("--extend")
+        regression_found = string_to_parse.find("The first")
+
+        if traceback_flag > -1:
+            #hg 1.9 and up only has --extend, which is branch aware
+            print "Using hg 1.9, we're branch aware! Let's explore that ancestor branch..."
+            subprocess.call(self.hgPrefix+["bisect","--extend"])
+        elif branch_unaware_flag > -1:
+            print "Not using hg 1.9 (not automatic) so you need to bisect again with the above changeset."
+        elif regression_found > -1:
+            print "Regression found using mozcommitbuilder " + progVersion + " on " + sys.platform + " at " + strftime("%Y-%m-%d %H:%M:%S", gmtime())
+            quit()
+
     def bisectRecurse(self, testcondition=None, args_for_condition=[]):
         #Recursively build, run, and prompt
         verdict = ""
 
-        if testcondition==None:
-            #Not using a test, interactive bisect begin!
-            self.buildAndRun()
-        elif testcondition != None:
-            #Using Jesse's idea: import any testing script and run it as the truth condition
+        try:
             self.build()
+        except Exception:
+            print "This build failed!"
+            verdict = "skip"
+
+        if verdict == "skip":
+            pass
+        elif testcondition==None:
+            #Not using a test, interactive bisect begin!
+            self.run()
+        else:
+            #Using Jesse's idea: import any testing script and run it as the truth condition
             args_to_pass = [self.objdir] + args_for_condition
 
             if hasattr(testcondition, "init"):
@@ -272,31 +296,27 @@ class Builder():
             if verdict != "bad" and verdict != "good":
                 verdict = "bad" if verdict else "good"
 
-        if verdict != 'good' and verdict !='bad':
-            while verdict != 'good' and verdict != 'bad' and verdict != 'b' and verdict != 'g':
-                verdict = raw_input("Was this commit good or bad? (type 'good' or 'bad' and press Enter): ")
+        while verdict not in ["good", "bad", "skip"]:
+            verdict = raw_input("Was this commit good or bad? (type 'good', 'bad', or 'skip'): ")
+            if verdict == 'g':
+                verdict = "good"
+            if verdict == 'b':
+                verdict = "bad"
+            if verdict == 's':
+                verdict = "skip"
 
-        #do hg bisect --good or --bad depending on whether it's good or bad
-        retval = 0;
-        if verdict == 'good':
-            retval = captureStdout(self.hgPrefix+["bisect","--good"])
-        else:
-            retval = captureStdout(self.hgPrefix+["bisect","--bad"])
+        # do hg bisect --good, --bad, or --skip
+        verdictCommand = self.hgPrefix+["bisect","--"+verdict]
+        print " ".join(verdictCommand)
+        retval = captureStdout(verdictCommand)
 
-        print str(retval)
         string_to_parse = str(retval)
-        traceback_flag = string_to_parse.find("Not all ancestors")
-        if traceback_flag > -1:
-            print "You need to re-run the bisector again using the changeset they give you."
+        print string_to_parse
 
-        # HACK
-        if retval[1] == 'h': #if retval starts with "the" then we can quit
-            quit()
-        elif retval[1] == 'e': #if retval starts with "testing" then it needs to keep going
+        self.check_done(string_to_parse)
+
+        if retval.startswith("Testing changeset"):
             print "\n"
-        else:
-            print "Something went wrong! :("
-            quit()
 
         self.bisectRecurse(testcondition=testcondition, args_for_condition=args_for_condition)
 
@@ -420,9 +440,14 @@ def cli():
     group1.add_option("-j", "--cores", dest="cores", default=cpuCount(),
                                         help="Max simultaneous make jobs (default: %default, the number of cores on this system)",
                                         metavar="[numjobs]")
+    group1.add_option("-m", "--mozconfig", dest="mozconf",
+                                        help="external mozconfig if so desired",
+                                        metavar="[mozconf path]", default=False)
+
     group1.add_option("-f", "--freshtrunk", action = "store_true", dest="makeClean", default=False,
                                         help="Delete old trunk and use a fresh one")
 
+
     group2 = OptionGroup(parser, "Bisector Options",
                                         "These are options for bisecting on changesets. Dates are retrieved from pushlog " \
                                         "and are not as reliable as exact changeset numbers.")
@@ -441,7 +466,7 @@ def cli():
                                         metavar="[changeset]")
 
     group3.add_option("-e", "--run", action="store_true", dest="run", default=False,
-                                        help="run the current build -- only works if already built")
+                                        help="run a single changeset")
 
     group4 = OptionGroup(parser, "Binary building options",
                                         "These are options for building binaries from a single changeset")
@@ -456,7 +481,6 @@ def cli():
                                         "Options for using an automated test instead of interactive prompting for bisection. " \
                                         "Please read documentation on how to write testing functions for this script.")
 
-
     group5.add_option("-c", "--condition", dest="condition", default=None, metavar="[condtest.py -opt1 -opt2]",
                                         help="External condition for bisecting. " \
                                              "Note: THIS MUST BE THE LAST OPTION CALLED.")
@@ -465,13 +489,10 @@ def cli():
                                         "Caution: use these options at your own risk.  "
                                         "They aren't recommended.")
 
-    group6.add_option("-r", "--repo", dest="repoURL",
+    group6.add_option("-R", "--repo", dest="repoURL",
                                         help="alternative mercurial repo to bisect NOTE: NEVER BEEN TESTED",
                                         metavar="valid repository url")
 
-    group6.add_option("-m", "--mozconfig", dest="mozconf",
-                                        help="external mozconfig if so desired NOTE: BROKEN RIGHT NOW",
-                                        metavar="path_to_mozconfig", default=False)
 
     parser.add_option_group(group1)
     parser.add_option_group(group2)
@@ -525,12 +546,10 @@ def cli():
         if options.repoURL:
             commitBuilder.repoURL = options.repoURL
             print "Using alternative repository "+options.repoURL
-        #if options.alternateMake:
-        #    commitBuilder.makeCommand = shlex.split(options.alternateMake)
 
+        conditionscript = None
         if options.condition:
             conditionscript = ximport.importRelativeOrAbsolute(options.condition)
-
         commitBuilder.bisect(options.good,options.bad, testcondition=conditionscript, args_for_condition=args_for_condition)
 
     # Should not get here.
